//********************************************************************
//*** ResourcePool/BatchCommand.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: BatchCommand.cc,v 1.4 2007/09/19 12:42:37 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_BATCHCOMMAND_HH
#include "BatchCommand.hh"
#endif

namespace fatalmind {

template<class CommandType>
BatchCommand<CommandType>::BatchCommand() 
: CommandType()
, list()
, error_iterator(list.end())
{
}

template<class CommandType>
BatchCommand<CommandType>::BatchCommand(const BatchCommand& rho) 
: CommandType(rho)
, list(rho.list)
, error_iterator(list.end())
{
}

template<class CommandType>
void BatchCommand<CommandType>::add(const CommandType& cmd) 
{
	list.push_back(gc_ptr<CommandType>(cmd.template clone<CommandType>()));
	error_iterator = list.end();
}

template<class CommandType>
void BatchCommand<CommandType>::execute(typename CommandType::pool_type::resource_t& res) 
{
	const_iterator end  = list.end();
	for (error_iterator = list.begin(); error_iterator != end; ++error_iterator) {
		(*error_iterator)->execute(res);
	}
	error_iterator = end; // no error hppend
}

template<class CommandType>
typename BatchCommand<CommandType>::const_iterator BatchCommand<CommandType>::getError() const
{
	return error_iterator;
}

template<class CommandType>
typename BatchCommand<CommandType>::const_iterator BatchCommand<CommandType>::end() const
{
	return list.end();
}

template<class CommandType>
typename BatchCommand<CommandType>::const_iterator BatchCommand<CommandType>::begin() const
{
	return list.begin();
}

template<class CommandType>
typename BatchCommand<CommandType>::member_t BatchCommand<CommandType>::back() const
{
	return list.back();
}

template<class CommandType>
bool BatchCommand<CommandType>::empty() const
{
	return list.empty();
}


		

} // namespace fatalmind
