//********************************************************************
//*** ResourcePool/BatchCommand.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: BatchCommand.hh,v 1.6 2007/10/05 19:46:48 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_BATCHCOMMAND_HH
#define RESOURCEPOOL_BATCHCOMMAND_HH

#include <vector>

#include "mm/gc_ptr.hh"
#include "util/Clone.hh"

namespace fatalmind {

/**
 * The class is a collection of Commands to be executed in sequential order.
 * Execution of commands stopps in case an exception is thrown.
 * BatchCommand's can also be nested.
 */
template<class CommandType>
class BatchCommand: public CommandType
{
	protected:
		typedef gc_ptr<CommandType> member_t;
		typedef std::vector<member_t> list_t;
	public:
		typedef typename list_t::const_iterator const_iterator;

		BatchCommand();
		BatchCommand(const BatchCommand&);
		virtual ~BatchCommand() {
		}

		virtual void add(const CommandType&);
		virtual void execute(typename CommandType::pool_type::resource_t&);

		/*
		 * returns an iterator to the command which did fail the last execution or
		 * end() if there was no error or no execution at all.
		 */
		const_iterator getError() const;
		
		const_iterator end() const;
		const_iterator begin() const;
		bool empty() const;
		member_t back() const;
		
	protected:
		virtual Clone* DoClone() const {
			return new BatchCommand(*this);
		}

		
	private:
		list_t list;
	protected:
		const_iterator error_iterator;

}; // class Batch

} // namespace fatalmind
#endif
