//********************************************************************
//*** ResourcePool/BatchCommand.t.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: BatchCommand.t.cc,v 1.4 2007/09/19 12:42:38 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "t/main.hh"
#include "util/Clone.hh"
#include "BatchCommand.hh"
#include "BatchCommand.cc"

using namespace fatalmind;

// same as in Execute.t.cc
#define THIS_IS_A_VERY_STRANGE_INCLUDE
#include "ResourcePool.t.cc"

typedef  Command<myType<DefaultThreadedModel> > cmdtype;
class MyTestCommandOK: public cmdtype {
	public:
		MyTestCommandOK(const int id = 0)
		: _called(0)
		, _id(id)
		{
		};
		void execute(Rec& res) {
			_called++;
		}
		int _called;
		int _id;
	protected:
		virtual Clone* DoClone() const {
			return new MyTestCommandOK(*this);
		}
};

class MyTestCommandEx: public MyTestCommandOK {
	public:
		MyTestCommandEx(int i = 0) 
		: MyTestCommandOK(i)
		{
		}
		void execute(Rec& res) {
			MyTestCommandOK::execute(res);
			throw Exception("hirsch");
		}
	protected:
		virtual Clone* DoClone() const {
			return new MyTestCommandEx(*this);
		}
};

typedef BatchCommand<MyTestCommandOK> batch_t;

void TEST_BatchCommand_OK() {
	MyTestCommandOK	cmdok(0);
	batch_t bc;

	Fac F("hirsch");	
	RP<> p(F);
	p.execute(cmdok);
	assertEqual(cmdok._called, 1);

	p.execute(bc); // empty batch command
	assertEqualT(int, cmdok._called, 1);
	assertTrue(bc.end() == bc.getError());

	bc.add(cmdok);
	p.execute(bc); // one member
	assertEqual(cmdok._called, 1); // operation was on clone!

	batch_t::const_iterator i = bc.begin();
	assertEqual((*i)->_called, 2);

	p.execute(bc); // one member
	i = bc.begin();
	assertEqual((*i)->_called, 3);

	bc.add(cmdok);
	p.execute(bc); // two member
	i = bc.begin();
	assertEqual((*i)->_called, 4);
	++i;
	assertEqual((*i)->_called, 2);
}

void TEST_BatchCommand_FAIL() {
	MyTestCommandOK	cmdok(0);
	MyTestCommandEx	cmdex(1);
	MyTestCommandOK	cmdok2(2);
	batch_t bc;

	Fac F("hirsch");
	RP<> p(F);

	bc.add(cmdok);
	bc.add(cmdex);
	bc.add(cmdok2);

	try {
		p.execute(bc);
		assertNotReach();
	} catch (...) {
	}

	batch_t::const_iterator i = bc.begin();
	assertEqual((*i)->_called, 2); // due to default MaxTry=2
	assertEqual((*i)->_id, 0);
	++i;
	assertEqual((*i)->_called, 2); // due to default MaxTry=2
	assertEqual((*i)->_id, 1);
	assertTrue(i == bc.getError());
	++i;
	assertEqual((*i)->_called, 0);
	assertEqual((*i)->_id, 2);
	++i;
	assertTrue(i == bc.end());
}
