//********************************************************************
//*** ResourcePool/SQL/RTSQLSelect.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelect.hh,v 1.12 2007/09/19 12:42:38 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLSELECT_HH
#define RESOURCEPOOL_SQL_RTSQLSELECT_HH

#include "mm/gc_ptr.hh"
#include "ResourcePool/SQL/RTSQLCommand.hh"
#include "ResourcePool/SQL/Resource.hh"
#include "ResourcePool/SQL/BindInterface.hh"
#include "ResourcePool/SQL/BindOutInterface.hh"
#include "ResourcePool/SQL/RTCommandParameters.hh"
#include "ResourcePool/SQL/RTSQLSelectRow.hh"

namespace fatalmind {
namespace SQL {

template<class TM> class RTSQLSelectRow_Impl;

/**
 * A command to issue a select statement against
 * an SQL database.
 */
template<class TM>
class RTSQLSelect
: public RTSQLCommand<TM>
, public BindInterface
, public BindOutInterface
{
	public:
		RTSQLSelect(const ResourcePool<ResourceType<TM> >& p, const std::string& sql);
		RTSQLSelect(const RTSQLSelect<TM>& rho);
		virtual ~RTSQLSelect();
		
		template<typename T>
		void bind(const unsigned int pos, const T& val, bool nul = false);

		template<typename T>
		void bindout(const unsigned int pos, T& val);

		bool isnull(const unsigned int pos) const;

		void execute(typename ResourceType<TM>::resource_t& hand);

		virtual bool process() = 0;

	private:
		virtual const Clone* getPlainSpecializedCommand() const;

		gc_ptr<RTSQLSelectRow_Impl<TM> > _realSQLExecute;	

		typedef typename SQLFactory<TM>::databaseType_t databaseType_t;
};


} // namespace SQL
} // namespace fatalmind
#endif
