//********************************************************************
//*** ResourcePool/SQL/RTSQLTransaction.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLTransaction.cc,v 1.3 2007/09/19 16:24:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "config.h"
#include "ResourcePool/SQL/ResourcePool.hh"
#include "ResourcePool/SQL/RTSQLTransaction.hh"
#include "ResourcePool/SQL/RTSQLTransaction_Impl.hh"

#include "ResourcePool/SQL/ConcreteWrapper.hh"

#include <sstream>
#include <typeinfo>

namespace fatalmind {
namespace SQL {

template<class PT, class TM, class RTSQLC>
class SQLTransactionWrapper
: public PT::command_t::Transaction
{
		typedef typename PT::command_t::Transaction super;
    public:
		SQLTransactionWrapper(RTSQLC& rtc, const typename RTSQLC::innercommand_t& cmd)
		: super(ConcreteWrapper<PT, ResourceType<TM> >(cmd))
		{
		}
};

template<class TM>
struct SQLTransactionCommandSelector {
	typedef RTSQLTransaction_Impl<TM> Impl;
	typedef RTSQLTransaction<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLTransaction_SpecializedImpl<
			  TM
			, RTSQLTransaction_Impl
			, PT
			, SQLTransactionWrapper<PT, TM, RTSQLTransaction<TM> >
			, RTSQLTransaction<TM>
		> Impl;
	};
};

template<class TM>
RTSQLTransaction<TM>::RTSQLTransaction(const ResourcePool<ResourceType<TM> >& p, const innercommand_t& a_cmd)
: _realSQLTransaction(
	RTSQLTransaction_Impl<TM>::template newRealCommand<
		SQLTransactionCommandSelector<TM>
	>(
		  innercommand_t::getFactory(p).getDatabaseType()
		, *this
		, a_cmd
	)
)
{
}

template<class TM>
RTSQLTransaction<TM>::RTSQLTransaction(const RTSQLTransaction<TM>& rho) 
: _realSQLTransaction(rho._realSQLTransaction->template clone<RTSQLTransaction_Impl<TM> >())
{
}

template<class TM>
RTSQLTransaction<TM>::~RTSQLTransaction() {
}

template<class TM>
Clone* RTSQLTransaction<TM>::DoClone() const {
	return new RTSQLTransaction<TM>(*this);
}


template<class TM>
void RTSQLTransaction<TM>::execute(typename ResourceType<TM>::resource_t& hand) {
	_realSQLTransaction->execute(hand);
}

template class RTSQLTransaction<DefaultThreadedModel >;

} // namespace SQL
} // namespace fatalmind
