//********************************************************************
//*** ResourcePool/SQL/SQLSelect.t.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.t.cc,v 1.16 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "t/main.hh"

using namespace std;
using namespace fatalmind;

class SQLSelect_test: public RP::Command::SQLSelect {
	public:
		SQLSelect_test(RP& p, const string& sql, int limit)
		: RP::Command::SQLSelect(p, sql)
		, _rows(0)
		, _limit(limit)
		{
		};

		SQLSelect_test(const SQLSelect_test& a) 
		: RP::Command::SQLSelect(a)
		, _rows(0)
		, _limit(a._limit)
		{
		}

		virtual ~SQLSelect_test() throw() {
		}

		bool process() {
			return ++_rows <= _limit;
		}

		int _rows;
		int _limit;

	protected:
		virtual SQLSelect_test* DoClone() const {
			return new SQLSelect_test(*this);
		}
};


void test_SQLSelect_init(RP& p)
{
	init(p);

	RP::Command::SQLExecute init(p, "insert into hirsch (id, ukn) values (?, ?)");
	init.bind<std::string>(1, "hohoho ' kasdfjs \" sdf");
	for (int i = 0; i < 600; ++i) {
		init.bind(0, i);
		p.execute(init);
	}
}


void test_SQLSelect_one(RP& p)
{
	string sql("select id, ukn from hirsch order by id");
	SQLSelect_test cmd(p, sql, 1000);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	p.execute(cmd);

	assertEqualT(int, col1, 599);
	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_all(RP& p)
{
	string sql("select id, ukn from hirsch");
	SQLSelect_test cmd(p, sql, 1000);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_ten(RP& p)
{
	string sql("select id, ukn from hirsch");
	SQLSelect_test cmd(p, sql, 10);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_one(RP& p)
{
	string sql("select id, ukn from hirsch where rownum <= 1");
	SQLSelect_test cmd(p, sql, 1);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		p.execute(cmd);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_SQLSelect_clone(RP& p) {
	string sql("select id, ukn from hirsch where id = ?");
	SQLSelect_test cmd(p, sql, 10);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	std::auto_ptr<SQLSelect_test> cmd2(cmd.clone<SQLSelect_test>());
	cmd.bind(0, 45); // available up to 599
	cmd2->bind(0, 46); // available up to 599

	p.execute(cmd);
	int x = cmd._rows;
	assertEqual(col1, 45);
	assertEqual(x, 1);
	assertEqual(cmd2->_rows, 0);
	p.execute(*cmd2);
	assertEqual(col1, 46);
	assertEqual(x, 1);
	assertEqual(cmd2->_rows, 1);
}

// TODO: test which does not match any record

void test_SQLSelect_cleanup(RP& p) {
	drop(p);
}
