//********************************************************************
//*** ResourcePool/mysql/MysqlException.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: MysqlException.cc,v 1.10 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"

#include <mysql/errmsg.h>

namespace fatalmind {

MysqlException::MysqlException(
	  const std::string& message
	, const int errorcode
	, const std::string& sql
):Exception(message) // virtual base class
, SQLException(message, mapMYSQLtoXOPEN(errorcode), sql, true)
, _errorcode(errorcode)
{
}

void MysqlException::throwit(
	  const int rv
	, const std::string& msg
	, const int errorcode
	, const std::string& sql
) {
	if (rv == CR_SERVER_LOST || rv == CR_SERVER_GONE_ERROR) {
		throw NFOMysqlException(msg, errorcode, sql);
	} else {
		std::string state(mapMYSQLtoXOPEN(errorcode));
		if (!state.compare(0,2, "08")) {
			throw MysqlException(msg, errorcode, sql);
		} else {
			throw NFOMysqlException(msg, errorcode, sql);
		}
	}
}

const char* MysqlException::mapMYSQLtoXOPEN(const int errorcode)
{
	switch(errorcode) {
		case 1058:
		case 1064:
		case 1136:
		case 1149: return "07000";
		case 1172: return "22000";
		case 1048:
		case 1062: return "23000";
		case 98: return "07001";
		case 99: return "02000";
	};
	return "08000"; //connection error
}

NFOMysqlException::NFOMysqlException(
	  const std::string& message
	, const int errorcode
	, const std::string& sql
): Exception(message) // virtual base class
, SQLException(message, mapMYSQLtoXOPEN(errorcode), sql, false)
, MysqlException(message, errorcode, sql)
, NoFailoverException(message)
{
}

} // namespace fatalmind
#endif


