//********************************************************************
//*** ResourcePool/oracle/CommandCommon.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: CommandCommon.cc,v 1.35 2007/09/14 18:27:39 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_COMMANDCOMMON_HH
#include "CommandCommon.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH
#include "ResourcePool.hh"
#endif

#include "oracleType.cc"
#include "ResourcePool/SQL/Bind.cc"

namespace fatalmind {
namespace oracle {

template<class TM, template<class> class SLMDC>
CommandCommon<TM, SLMDC>::CommandCommon(const std::string& SQL) 
: _SQL(SQL)
, _sth()
{
}

template<class TM, template<class> class SLMDC>
CommandCommon<TM, SLMDC>::~CommandCommon() {
}
/*
template<class TM, template<class> class SLMDC>
void CommandCommon<TM, SLMDC>::execute(oracleHandlesWithCache<TM>& hand) {
	StatementCache sthcache;
	_execute(hand, sthcache);
}

*/

template<class TM, template<class> class SLMDC>
void CommandCommon<TM, SLMDC>::prepare(oracleHandles& hand, StatementCache& sthcache, const oracleStatement& oraSQL) {
	_sth = sthcache.open(oraSQL, true);
	_sth->prepare(hand);
}

template<class TM, template<class> class SLMDC>
void CommandCommon<TM, SLMDC>::real_bind(oracleHandles& hand) {
	sword ret;
	_argst::const_iterator end = _args.end();
	int pos = 0;
	for (_argst::const_iterator i = _args.begin(); i != end; ++i) {
		++pos;
		OCIBind  *bnd1p = NULL;
		if ((*i).get() != 0) {
			if ((ret = OCIBindByPos(_sth->get()
				, &bnd1p
				, hand.getError()
				, pos 
				, (*i)->ptr()
				, (*i)->size()
				, (*i)->type()
				, (*i)->indptr()
				, (ub2 *)0
				, (ub2 *)0
				, 0
				, (ub4 *)0
				, OCI_DEFAULT)
			) != OCI_SUCCESS) {
				OracleExceptionFactory::throwit("OCIBindByPos", hand, ret, _SQL);
			}
		} else {
			OracleExceptionFactory::throwit("OCIBindByPos", "ORA", 1008, "not all params bound", _SQL);
		} 
	}
}

template<class TM, template<class> class SLMDC>
const std::string& CommandCommon<TM, SLMDC>::getSQL() const {
	return _SQL;
}

template class CommandCommon<DefaultThreadedModel, oracle::SelectListMetaDataCache >;
template class CommandCommon<DefaultThreadedModel, oracle::SelectListMetaDataNopCache >;


} // namespace oracle

template class
SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>;

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<char>
	(const unsigned int pos, const char& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<signed char>
	(const unsigned int pos, const signed char& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned char>
	(const unsigned int pos, const unsigned char& val, bool nul);

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<short int>
	(const unsigned int pos, const short int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<int>
	(const unsigned int pos, const int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<long int>
	(const unsigned int pos, const long int& val, bool nul);

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned short int>
	(const unsigned int pos, unsigned const short int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned int>
	(const unsigned int pos, unsigned const int& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<unsigned long int>
	(const unsigned int pos, unsigned const long int& val, bool nul);

// wchar_t, bool, not supported..

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<float>
	(const unsigned int pos, const float& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<double>
	(const unsigned int pos, const double& val, bool nul);
template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<long double>
	(const unsigned int pos, const long double& val, bool nul);

template void SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>::bind<std::string>
	(const unsigned int pos, const std::string& val, bool nul);


} // namespace fatalmind

