//********************************************************************
//*** ResourcePool/oracle/Factory.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Factory.cc,v 1.19 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_FACTORY_HH
#include "Factory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIENVHANDLE_HH
#include "OCIEnvHandle.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTION_HH
#include "OracleException.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_CREATEXCEPTION_HH
#include "ResourcePool/CreateException.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLECONNECT_HH
#include "lex.oracle_connect.hh"
#endif

#ifndef OCI_ORACLE
#include <oci.h>
#endif

#include <string>

namespace fatalmind {

template<class TM, template<class> class SLMDC>
oracleFactory<TM, SLMDC>::oracleFactory(const std::string& service_name
            , const std::string& user
            , const std::string& pass
)
: _service_name(service_name)
, _user(user)
, _pass(pass)
, _mode(TM::isMultiThreaded?OCI_THREADED:OCI_DEFAULT)
, _env()
, _slmdc(new typename SLMDC<TM>::type(1000))
{
}

template<class TM, template<class> class SLMDC>
oracleFactory<TM, SLMDC>::oracleFactory(const std::string& logon)
: _mode(TM::isMultiThreaded?OCI_THREADED:OCI_DEFAULT)
, _env()
, _slmdc(new typename SLMDC<TM>::type(1000))
{
	oracle_connect oc(logon);
	_service_name = oc.getService();
	_user         = oc.getUser();
	_pass         = oc.getPass();
}


template<class TM, template<class> class SLMDC>
oracleFactory<TM, SLMDC>::~oracleFactory()
{
}

template<class TM, template<class> class SLMDC>
oracleFactory<TM, SLMDC>::oracleFactory(const oracleFactory& rho)
: _service_name(rho._service_name)
, _user(rho._user)
, _pass(rho._pass)
, _mode(rho._mode)
, _env(rho._env)
, _slmdc(rho._slmdc)
{
}

template<class TM, template<class> class SLMDC>
Clone* oracleFactory<TM, SLMDC>::DoClone() const
{
	return new oracleFactory<TM, SLMDC>(*this);
}

class oracleCreateException: public OracleException, public CreateException {
	public:
		oracleCreateException(const OracleException& e)
		: Exception(e)
		, SQLException(e)
		, OracleException(e)
		, CreateException(e.message())
		{
		}
};

template<class TM, template<class> class SLMDC>
oracleResource<TM>* oracleFactory<TM, SLMDC>::create_resource() const
{
	try {
		if (_env.get() == 0) {
			_env.reset(new oracle::OCIEnvHandle(_mode));
		}
		return new oracleResource<TM>(_service_name, _user, _pass, *_env, _slmdc);
	} catch (const OracleException& e) {
		throw oracleCreateException(e);
	}
}
} // namespace fatalmind
