//********************************************************************
//*** ResourcePool/oracle/StatementCache.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: StatementCache.hh,v 1.10 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH
#define RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLEFWD_HH
#include "OCIHandleFwd.hh"
#endif

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLESTATEMENT_HH
#include "oracleStatement.hh"
#endif

#include <map>
#include <string>

namespace fatalmind {
namespace oracle {

class StatementCache {
	public:
		typedef gc_ptr<OCIStatementHandle> handle_t;

		StatementCache(unsigned int maxsz = 65535, StatementCache* parent = 0);
		~StatementCache();

        // to ensure the handle is not deleted (expired from the cache)
        // while it is still used by the client, the client has to
        // assign it to a oracle::StatementCache::handle_t
        // the return is passed as reference to avoid an additional temporary
        // TODO: hide so that only oracleHandles can use it!
		handle_t& open(const oracleStatement& sql, bool parentcache = false);

	private:
		handle_t* check(const oracleStatement& sql);

		void expire();
		void chksize();

		unsigned int _maxsz;
		StatementCache* _parent;

		typedef std::pair<oracleStatement, handle_t> element_t;
		typedef std::map<oracleStatement, handle_t> map_t;
		map_t _map;
	// hidden stuff
		StatementCache(const StatementCache&);
		StatementCache* operator=(const StatementCache&);
};

}
}
#endif
