//********************************************************************
//*** Thread/LockStatistics.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: LockStatistics.hh,v 1.12 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_THREAD_LOCKSTATISTICS_HH
#define INCLUDED_THREAD_LOCKSTATISTICS_HH

#ifndef INCLUDED_THREAD_LOCKSTATISTICSFWD_HH
#include "LockStatisticsFwd.hh"
#endif

#ifndef POSIX_TIMERFWD_HH
#include "POSIX/timerFwd.hh"
#endif

#ifndef INCLUDED_THREAD_LOCKFWD_HH
#include "LockFwd.hh"
#endif

#include <iosfwd>

namespace fatalmind {

class LockUsageStatistics {
	public:
		inline LockUsageStatistics()
		: _lock(0)
		, _reclev(0)
		, _maxreclev(0)
		{
		}
		
		inline void stat_lock() {
			++_lock;
			++_reclev;
		}

		inline void stat_unlock() {
			if (_reclev > _maxreclev) {
				_maxreclev = _reclev;
			}
			--_reclev;
		}

	friend std::ostream& operator<<(std::ostream& o, const LockUsageStatistics& s);
	private:
		int _lock;
		int _reclev;
		int _maxreclev;
};

} // namespace fatalmind

#ifndef POSIX_TIME_HH
#include "POSIX/time.hh"
#endif

namespace fatalmind {

class LockTimingStatistics {
	public:
		inline LockTimingStatistics()
		: _waittime()
		{};
		typedef timer<MultiThreadedModel<LockStatistics<NopLockUsageStatistics,NopLockTimingStatistics> >, SingleThreadedModel<LockStatistics<NopLockUsageStatistics, NopLockTimingStatistics> > > _timer_t;
		POSIX::timespec<MultiThreadedModel<LockStatistics<NopLockUsageStatistics, NopLockTimingStatistics> > > _waittime;
	friend
		std::ostream& operator<<(std::ostream& o, const LockTimingStatistics& s);
};

} // namespace fatalmind

#endif
